import time
from datetime import datetime

import requests
from bs4 import BeautifulSoup

import jieba
import jieba.analyse
#jieba.set_dictionary('dict.txt.big.txt')
from snownlp import SnowNLP

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm as cm

#------------------------ 爬新聞
count = 1
page = 2
delay = 0.2
news_group = []

for i in range(1,page):
	time.sleep(delay)
	res = requests.get("http://www.appledaily.com.tw/realtimenews/section/new/"+str(i))
	soup = BeautifulSoup(res.text,"html.parser")
	day = soup.find("h1","dddd").text.replace(" ","")

	for item in soup.select(".rtddt"):
		# 取得新聞標題、時間、類別
		news_title = item.find("h1").text
		news_popular = int(news_title[news_title.rfind("(")+1 : -1])
		news_time = item.find("time").text
		news_category = item.find("h2").text
		
		# 取得新聞簡述
		time.sleep(delay)
		news_url = "http://www.appledaily.com.tw/"+item.find('a')['href']
		res2 = requests.get(news_url)
		soup2 = BeautifulSoup(res2.text,"html.parser")
		news_content = soup2.find("meta",{"name" : "description"})	
		if news_content:
			news_content = news_content["content"]
		else:
			news_content = ""
			
		# 將新聞放進news_grop裡
		news_group.append({"標題":news_title, "時間":day+" "+news_time, "類別":news_category, "內文":news_content, "人氣":news_popular})
		print("======[",count,"]=========")
		print(news_title,"【",news_category,"】")
		print(day,news_time)
		
		print(news_content)
		
		if news_content != "":
			# Jieba文本分析
			print("============Jieba============") 
			seglist = jieba.cut(news_content, cut_all=False)
			print("斷詞：") 
			print("/ ".join(seglist)) 
			tags = jieba.analyse.extract_tags(news_content, 3)
			print("三個關鍵字：")
			print("/ ".join(tags))
			# Snownlp文本分析
			print("============SnowNLP============") 
			s = SnowNLP(news_content)
			print("斷詞：")
			print("/ ".join(s.words))  
			print("三個關鍵字：")
			print("/ ".join(s.keywords(3))) 
			print("三個摘要：")
			print("/ ".join(s.summary(3))) 
			print("情感分析：")
			print(s.sentiments)
		count += 1

#------------------------ 資料視覺化 (時間軸 vs. 人氣)
# 統計有多少類別需要上色
# category = ["焦點","熱門","爆社","動物","副刊","3C","影片","火線","體育","壹周刊","媒陣","娛樂","時尚","生活","社會","國際","財經","地產","政治","論壇"]
category = []
have_label = []
for news in news_group:
	if news["類別"] not in category:
		print(len(category),news["類別"])
		category.append(news["類別"])
		have_label.append(0)

# 平均分配顏色給類別
colors = cm.rainbow(np.linspace(0, 1, len(category)))
colors_map = dict()
for i in range(len(category)):
	colors_map[category[i]] = colors[i]

# 在圖表上畫點 (x,y) = (距離現在的時間,人氣)
latest_time = datetime.strptime(news_group[0]["時間"], "%Y/%m/%d %H:%M")
for news in news_group:
	news_time = datetime.strptime(news["時間"], "%Y/%m/%d %H:%M")
	x = int((latest_time-news_time).seconds//60)
	y = int(news["人氣"])
	if have_label[category.index(news["類別"])] == 1:
		plt.scatter([x], [y], color=colors_map[news["類別"]])
	else:
		plt.scatter([x], [y], color=colors_map[news["類別"]], 
					label=str(category.index(news["類別"])))
		have_label[category.index(news["類別"])] = 1

# 標示圖表其他資訊並顯示		
plt.title('timeline vs. popular')
plt.xlabel('new <-- old(minute)')
plt.ylabel('popular')
plt.legend()
plt.show()